//********************************************************************
//*** ResourcePool/SQL/RTSQLSelect.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLSelect.cc,v 1.9 2005/03/19 15:30:17 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "ResourcePool/SQL/ResourcePool.hh"
#include "ResourcePool/SQL/RTSQLSelect.hh"
#include "ResourcePool/SQL/RTSQLSelectRow_Impl.hh"

namespace fatalmind {
namespace SQL {

template<class C, class TM, class RTSQLC>
class SQLSelectWrapper
: public C
{
	public:
		SQLSelectWrapper(RTSQLC& rtc, const std::string& sql)
		: C(sql)
		, rtc_(rtc)
		{
		}

		SQLSelectWrapper<C, TM, RTSQLC>* clone() const {
			return new SQLSelectWrapper<C, TM, RTSQLC>(*this);
		}

		virtual bool process() {
			return rtc_.process();
		}
	private:
		RTSQLC& rtc_;
};

template<class TM>
struct SQLSelectCommandSelector {
    typedef RTSQLSelectRow_Impl<TM> Impl;
    typedef RTSQLSelect<TM> RTC;

	template<class PT>
	struct Specialized {
		typedef RTSQLSelectRow_SpecializedImpl<
			  TM
			, RTSQLSelectRow_Impl
			, PT
			, SQLSelectWrapper<typename PT::command_t::SQLSelect, TM, RTSQLSelect<TM> >
			, RTSQLSelect<TM>
		> Impl;
	};
};

template<class TM>
RTSQLSelect<TM>::RTSQLSelect(const ResourcePool<ResourceType<TM> >& p, const std::string& sql)
: _realSQLExecute(
    RTSQLExecute_Impl<TM>::template newRealCommand<
          SQLSelectCommandSelector<TM>
    >(
		  Command<ResourceType<TM> >::getFactory(p).getDatabaseType()
        , *this
        , sql
    )
)
{
}

template<class TM>
RTSQLSelect<TM>::RTSQLSelect(const RTSQLSelect<TM>& rho)
: _realSQLExecute(_realSQLExecute->clone())
{
}

template<class TM>
RTSQLSelect<TM>::~RTSQLSelect() {
}

template<class TM>
template<class T>
void RTSQLSelect<TM>::bind(const unsigned int pos, const T& val, bool nul) {
	_realSQLExecute->bind(pos, val, nul);
}

template<class TM>
template<class T>
void RTSQLSelect<TM>::bindout(const unsigned int pos, T& val) {
	_realSQLExecute->bindout(pos, val);
}

template<class TM>
void RTSQLSelect<TM>::execute(typename ResourceType<TM>::plainresource_t& hand) {
	_realSQLExecute->execute(hand);
}

template class RTSQLSelect<SingleThreadedModel<> >;

template void RTSQLSelect<SingleThreadedModel<> >::bind<short>         (const unsigned int pos, const short& val, bool nul);
template void RTSQLSelect<SingleThreadedModel<> >::bind<int>           (const unsigned int pos, const int& val, bool nul);
template void RTSQLSelect<SingleThreadedModel<> >::bind<long>          (const unsigned int pos, const long& val, bool nul);
template void RTSQLSelect<SingleThreadedModel<> >::bind<double>        (const unsigned int pos, const double& val, bool nul);
template void RTSQLSelect<SingleThreadedModel<> >::bind<float>         (const unsigned int pos, const float& val, bool nul);
template void RTSQLSelect<SingleThreadedModel<> >::bind<std::string>   (const unsigned int pos, const std::string& val, bool nul);

template void RTSQLSelect<SingleThreadedModel<> >::bindout<short>      (const unsigned int pos, short& val);
template void RTSQLSelect<SingleThreadedModel<> >::bindout<int>        (const unsigned int pos, int& val);
template void RTSQLSelect<SingleThreadedModel<> >::bindout<long>       (const unsigned int pos, long& val);
template void RTSQLSelect<SingleThreadedModel<> >::bindout<double>     (const unsigned int pos, double& val);
template void RTSQLSelect<SingleThreadedModel<> >::bindout<float>      (const unsigned int pos, float& val);
template void RTSQLSelect<SingleThreadedModel<> >::bindout<std::string>(const unsigned int pos, std::string& val);

} // namespace SQL
} // namespace fatalmind
