//********************************************************************
//*** ResourcePool/TaskRunner/TaskRunnerPool.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: TaskRunnerPool.cc,v 1.5 2005/04/10 13:43:54 mws Exp $
//********************************************************************

#include "ResourcePool/TaskRunner/TaskRunnerPool.hh"

// we instantiate the respective ResourcePool here
#include "ResourcePool/ResourcePool.cc"
#include "ResourcePool/Execute.cc"
#include "ResourcePool/Command.cc"

using namespace std;

namespace fatalmind {

/*
 * 
 * CleanUpTask
 *
 */

namespace TaskRunnerPoolInternals {

class ReleaseTaskRunner {
	public:
		ReleaseTaskRunner(TaskRunnerPool& trp
			, TaskRunner& tr)
		: _trp(trp)
		, _tr(tr)
		{
		}
		
		~ReleaseTaskRunner()
		{
			_trp.free(_tr);
		}
	
	private:
		TaskRunnerPool& _trp;
		TaskRunner&     _tr;
};
	
class CleanUpTask: public fatalmind::Task {
	public:
		CleanUpTask(TaskRunnerPool& trp, TaskRunner& tr, CommonPtr<Task> tsk)
		: _trp(trp)
		, _tr(tr)
		, _tsk(tsk)
		{
		}
		
		virtual void run() {
			ReleaseTaskRunner(_trp, _tr);	
			_tsk->run();
			//_tr.run(_tsk);
		}	

	private:
		TaskRunnerPool& _trp;
		TaskRunner&     _tr;
		CommonPtr<Task> _tsk;
};

} // namespace TaskRunnerPoolInternals

/*
 *
 * TaskRunnerPool
 *
 */

TaskRunnerPool::TaskRunnerPool(const ResourcePoolOptions& rpo)
: _factory()
, _pool(_factory, rpo)
{
}

TaskRunnerPool::~TaskRunnerPool()
{
}

void TaskRunnerPool::runDetached(CommonPtr<Task> tsk) 
{
	TaskRunner& tr = _pool.get();
	auto_ptr<TaskRunnerPoolInternals::CleanUpTask> cut(
		new TaskRunnerPoolInternals::CleanUpTask(*this, tr, tsk)
	);
	tr.runDetached(cut);
}

TaskRunner::ID TaskRunnerPool::run(CommonPtr<Task> tsk) 
{
	TaskRunner& tr = _pool.get();
	auto_ptr<TaskRunnerPoolInternals::CleanUpTask> cut(
		new TaskRunnerPoolInternals::CleanUpTask(*this, tr, tsk)
	);
	return tr.run(cut);
}

void TaskRunnerPool::free(TaskRunner& tr)
{
	_pool.free(tr);
}

} // namespace fatalmind
